package no.ntnu.ime.xsltgenerator.client;

import java.util.List;
import java.util.Map;

import no.ntnu.ime.xsltgenerator.client.models.SaxParserResult;
import no.ntnu.ime.xsltgenerator.client.services.XMLSourceParserService;
import no.ntnu.ime.xsltgenerator.client.services.XMLSourceParserServiceAsync;
import no.ntnu.ime.xsltgenerator.client.ui.GeneratorDialog;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.xml.client.XMLParser;

public class SourceSelector extends Composite {
	private TextBox txtbxSourceuri;
	private VerticalPanel verticalPanel;
	private XMLSourceParserServiceAsync xmlSourceParser = GWT.create(XMLSourceParserService.class);
	private HTML messageBox;

	public SourceSelector() {

		verticalPanel = new VerticalPanel();
		verticalPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
		initWidget(verticalPanel);

		Label xmlSourceSelectorHeader = new HTML("<h2>Select your XML Source</h2>");
		verticalPanel.add(xmlSourceSelectorHeader);
		
		messageBox = new HTML("error", true);
		messageBox.setStyleName("error");
		verticalPanel.add(messageBox);
		messageBox.setVisible(false);

		Label lblUrlToXml = new Label("URL to xml file:");
		verticalPanel.add(lblUrlToXml);

		txtbxSourceuri = new TextBox();
		verticalPanel.add(txtbxSourceuri);
		txtbxSourceuri.setText("file:///home/klette/meldal.xml");
		txtbxSourceuri.setSize("90%", "16px");

		Button submitButton = new Button();
		submitButton.setStyleName("btn", true);
		submitButton.setStyleName("primary", true);
		verticalPanel.add(submitButton);
		submitButton.addClickHandler(new ClickHandler() {
		private final ProgressBar pb = new ProgressBar(5, ProgressBar.SHOW_TEXT);

			@Override
			public void onClick(ClickEvent event) {
				if (xmlSourceParser == null) {
					xmlSourceParser = GWT.create(XMLSourceParserService.class);
				}
				messageBox.setVisible(false);

				pb.setText("Validerer XML-kilde");
				pb.setProgress(pb.getProgress() + 20);
				xmlSourceParser.isValidSourceURI(txtbxSourceuri.getText(), new AsyncCallback<Boolean>() {

					@Override
					public void onSuccess(Boolean isValidXmlSource) {
						if (!isValidXmlSource) {
							
							messageBox.setText("Beklager, men XML-kilden du tastet inn returnerer et ugyldig XML-dokument. Vennligst dobbeltsjekk verdien og prøv på nytt.");
							messageBox.setVisible(true);
							
						} else {

							pb.setText("Henter XML-dokumentet for prosessering");
							pb.setProgress(pb.getProgress() + 20);

							RootPanel.get().add(pb);
							analyzeXMLDocument(pb, txtbxSourceuri.getText());
								
						}
					}

					@Override
					public void onFailure(Throwable caught) {
						messageBox.setText("Beklager, men XML-kilden du tastet inn ser ikke ut til å være gyldig. Vennligst dobbeltsjekk verdien og prøv på nytt.");
						messageBox.setVisible(true);
					}
				});

			}
		});
		submitButton.setText("Parse XML Source");

		HTML helpText = new HTML("<strong>Information:</strong><br /><dl>" + "<dt>Supported protocols:</dt>"
				+ "<dd>HTTP (http://www.example.org/source.xml)</dd>" + "<dd>HTTPS (https://www.example.org/source.xml)</dd>"
				+ "<dd>FTP (ftp://ftp.example.org/source.xml)</dd>"
				+ "<dd>FTP (with login) (ftp://username:password@ftp.example.org/source.xml)</dd>" 
				+ "<dd>File (file:///srv/import/source.xml)</dd>"
				+ "</dl>");
		helpText.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
		verticalPanel.add(helpText);

	}
	
	private void analyzeXMLDocument(final ProgressBar pb, final String sourceAdress) {
		pb.setProgress(pb.getProgress() + 20);
		pb.setText("Analyserer XML-dokumentet (" + sourceAdress + ")");
		xmlSourceParser.analyzeXMLFromURL(sourceAdress, new AsyncCallback<Map<String, List<SaxParserResult>>>() {
			
			@Override
			public void onSuccess(final Map<String, List<SaxParserResult>> parsingResult) {
				pb.setProgress(pb.getProgress() + 20);
				pb.setText("Lager tre-representasjon av XML-dokumentet");
				generateManualConvertorDialog(pb, sourceAdress, parsingResult);
			}

			@Override
			public void onFailure(Throwable caught) {
				Window.alert(caught.getLocalizedMessage());

			}
		});
	}
	private void generateManualConvertorDialog(final ProgressBar pb, final String url, final Map<String, List<SaxParserResult>> parsingResult) {
		xmlSourceParser.getXMLContentFromURL(url, new AsyncCallback<String>() {

			@Override
			public void onSuccess(String result) {
				
				pb.setProgress(pb.getProgress() + 20);
				pb.setText("Lager tre-representasjon av XML-dokumentet");
				GeneratorDialog personModel = new GeneratorDialog(parsingResult, XMLParser.parse(result));
				RootPanel.get().clear();
				RootPanel.get().add(personModel);
			}

			@Override
			public void onFailure(Throwable caught) {
				messageBox.setText("Beklager, noe gikk galt.");
				messageBox.setVisible(true);
			}
		});
	}

}
