package no.ntnu.ime.xsltgenerator.client.ui;

import java.util.List;
import java.util.Map;

import no.ntnu.ime.xsltgenerator.client.models.SaxParserResult;
import no.ntnu.ime.xsltgenerator.client.models.SaxParserResultData;
import no.ntnu.ime.xsltgenerator.client.services.XMLSourceParserService;
import no.ntnu.ime.xsltgenerator.client.services.XMLSourceParserServiceAsync;

import com.google.gwt.cell.client.TextCell;
import com.google.gwt.core.client.GWT;
import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import com.google.gwt.user.cellview.client.CellTable;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TabPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.xml.client.Document;

public class GeneratorDialog extends Composite {

	private TabPanel tabs;
	private VerticalPanel verticalPanel;
	private HTML htmlNewHtml;

	private ManualConversionPanel manualGeneratationPanel;

	public GeneratorDialog() {
	}

	/**
	 * @wbp.parser.constructor
	 */
	public GeneratorDialog(
			final Map<String, List<SaxParserResult>> parsingResult,
			final Document sourceXML) {

		tabs = new TabPanel();
		initWidget(tabs);
		tabs.setWidth("100%");

		verticalPanel = new VerticalPanel();
		tabs.add(verticalPanel, "Automatisk generert transformasjon", false);
		verticalPanel.setSize("100%", "100%");
		tabs.selectTab(0);

		addDataSet(verticalPanel, "Uthentet person", "person", parsingResult);
		addDataSet(verticalPanel, "Uthentet gruppe", "group", parsingResult);
		addDataSet(verticalPanel, "Uthentet organisasjon", "organization",
				parsingResult);
		addDataSet(verticalPanel, "Uthentet medlemskap", "relation",
				parsingResult);

		final XMLSourceParserServiceAsync backend = GWT
				.create(XMLSourceParserService.class);
		backend.getXlstForDataset("/foo/bar", parsingResult,
				new AsyncCallback<String>() {

					@Override
					public void onFailure(final Throwable caught) {
						htmlNewHtml = new HTML("<pre>" + caught.getMessage()
								+ "</pre>");

					}

					@Override
					public void onSuccess(final String result) {
						htmlNewHtml = new HTML(
								"<h2>Generert XSLT</h2>"
										+ "<pre class=\"prettyprint\">"
										+ SafeHtmlUtils.htmlEscape(result)
										+ "</pre>"
										+ "<script type=\"text/javascript\">prettyPrint();</script>",
								true);
						verticalPanel.add(htmlNewHtml);
						prettyPrint();

						backend.attemptConversion(result, sourceXML.toString(),
								new AsyncCallback<String>() {

									@Override
									public void onFailure(Throwable caught) {
										// TODO Auto-generated method stub

									}

									@Override
									public void onSuccess(String result) {
										HTML gen = new HTML(
												"<h2>XML etter transformasjon</h2>"
														+ "<pre class=\"prettyprint\">"
														+ SafeHtmlUtils
																.htmlEscape(result)
														+ "</pre>"
														+ "<script type=\"text/javascript\">prettyPrint();</script>",
												true);
										verticalPanel.add(gen);
										prettyPrint();
										

									}
								});

					}
				});
		GWT.log("Genering manial conversion panel");
		manualGeneratationPanel = new ManualConversionPanel(sourceXML);
		tabs.add(manualGeneratationPanel, "Manuell konvertering", false);

	}

	private void addDataSet(VerticalPanel container, String headerText,
			String lookupKey,
			final Map<String, List<SaxParserResult>> parsingResult) {
		
		HTML header = new HTML("<h2>" + headerText + "</h2>");
		container.add(header);

		CellTable<SaxParserResultData> t = new CellTable<SaxParserResultData>();
		t.setStyleName("zebra-striped", true);
		TextCell vcell = new TextCell();
		Column<SaxParserResultData, String> typeColumn = new Column<SaxParserResultData, String>(vcell) {

			@Override
			public String getValue(SaxParserResultData object) {
				return object.getType();
			}
		};
		TextCell dcell = new TextCell();
		Column<SaxParserResultData, String> valueColumn = new Column<SaxParserResultData, String>(dcell) {

			@Override
			public String getValue(SaxParserResultData object) {
				return object.getContent();
			}
		};
		TextCell pcell = new TextCell();
		Column<SaxParserResultData, String> pathColumn = new Column<SaxParserResultData, String>(pcell) {

			@Override
			public String getValue(SaxParserResultData object) {
				return object.getQuery();
			}
		};
		t.addColumn(typeColumn, "Felttype");
		t.addColumn(valueColumn, "Verdi");
		t.addColumn(pathColumn, "XPath-utrykk");
		
		

		//container.add(dataTable);

		if (parsingResult.containsKey(lookupKey)) {
			List<SaxParserResult> list = parsingResult.get(lookupKey);
			for (SaxParserResult res : list) {
				
				String xpathHeader = "<h3>" + res.getXPath() + "</h3>";
				header.setHTML(header.getHTML() + xpathHeader);
				List<SaxParserResultData> entries = res.getEntries();
				t.setRowData(entries);
			}

		}
		container.add(t);
	}

	public static native void prettyPrint()/*-{
		prettyPrint();
	}-*/;
}
