package no.ntnu.ime.xsltgenerator.server;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import no.ntnu.ime.libxsltgenerator.XSLTGenerator;
import no.ntnu.ime.libxsltgenerator.exceptions.ValidSourceException;

import no.ntnu.ime.xsltgenerator.client.models.SaxParserResult;
import no.ntnu.ime.xsltgenerator.client.models.SaxParserResultData;
import no.ntnu.ime.xsltgenerator.client.services.XMLSourceParserService;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

public class XMLSourceParserServiceImpl extends RemoteServiceServlet implements
		XMLSourceParserService {

	private static final long serialVersionUID = 1L;
	private XSLTGenerator generator = new XSLTGenerator();

	@Override
	public String getXMLContentFromURL(String url) {
		System.out.println(url);
		try {
			File file = generator.retrieveXMLSource(url);
			FileReader fileReader = new FileReader(file);
			BufferedReader bufferedReader = new BufferedReader(fileReader);
			StringBuilder stringBuilder = new StringBuilder();
			String line = null;
			while ((line = bufferedReader.readLine()) != null) {
				stringBuilder.append(line);
			}
			return stringBuilder.toString();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	@Override
	public Map<String, List<SaxParserResult>> analyzeXMLFromURL(String url) {
		try {
			Map<String, List<SaxParserResult>> results = new HashMap<String, List<SaxParserResult>>();
			Map<String, List<no.ntnu.ime.libxsltgenerator.models.SaxParserResult>> rawParsed = generator.parseXMLSource(url);
			for (Entry<String, List<no.ntnu.ime.libxsltgenerator.models.SaxParserResult>> entry : rawParsed.entrySet()){
				List<SaxParserResult> list = new ArrayList<SaxParserResult>();
				
				convertToGwtModel(entry, list);
				results.put(entry.getKey(), list);
			}
			return results;
		} catch (Exception e){
			throw new RuntimeException(e);
		}
	}

	private void convertToGwtModel(
			Entry<String, List<no.ntnu.ime.libxsltgenerator.models.SaxParserResult>> entry,
			List<SaxParserResult> list) {
		for (no.ntnu.ime.libxsltgenerator.models.SaxParserResult spr : entry.getValue()){
			List<SaxParserResultData> sprdList = new ArrayList<SaxParserResultData>();
			for (no.ntnu.ime.libxsltgenerator.models.SaxParserResultData foo : spr.getEntries()){
				sprdList.add(new SaxParserResultData(foo.getQuery(), foo.getType(), foo.getContent()));
			}
			list.add(new SaxParserResult(spr.getXpath(), spr.getData(), sprdList));
		}
	}
	
	private void convertFromGwtModel(
			Entry<String, List<SaxParserResult>> entry,
			List<no.ntnu.ime.libxsltgenerator.models.SaxParserResult> list) {
		for (SaxParserResult spr : entry.getValue()){
			List<no.ntnu.ime.libxsltgenerator.models.SaxParserResultData> sprdList = new ArrayList<no.ntnu.ime.libxsltgenerator.models.SaxParserResultData>();
			for (SaxParserResultData foo : spr.getEntries()){
				sprdList.add(new no.ntnu.ime.libxsltgenerator.models.SaxParserResultData(foo.getContent(), foo.getType(), foo.getQuery()));
			}
			list.add(new no.ntnu.ime.libxsltgenerator.models.SaxParserResult(spr.getXPath(), spr.getData(), sprdList));
		}
	}

	@Override
	public boolean isValidSourceURI(String sourceAdress)
			throws RuntimeException {
		try {
			return generator.isValidSourceURI(sourceAdress);
		} catch (ValidSourceException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	@Override
	public String getXlstForDataset(String string,
			Map<String, List<SaxParserResult>> parsingResult) {
		try {
		Map<String, List<no.ntnu.ime.libxsltgenerator.models.SaxParserResult>> results = new HashMap<String, List<no.ntnu.ime.libxsltgenerator.models.SaxParserResult>>();
		for (Entry<String, List<SaxParserResult>> entry : parsingResult.entrySet()){
			List<no.ntnu.ime.libxsltgenerator.models.SaxParserResult> list = new ArrayList<no.ntnu.ime.libxsltgenerator.models.SaxParserResult>();
			
			convertFromGwtModel(entry, list);
			results.put(entry.getKey(), list);
		}

		return generator.getXlstForDataset(string, results);
	} catch (Exception e){
		throw new RuntimeException(e);
	}
	}

	@Override
	public String attemptConversion(String result, String string) {
		try {
			File tempFile = File.createTempFile("conversion", "xml");
			FileWriter fw = new FileWriter(tempFile);
			fw.write(string);
			fw.flush();
			fw.close();
			return generator.attemptConversion(result, tempFile);
		} catch (Exception e){
			return e.toString();
		}
	}

}
