package no.ntnu.ime.xmlconverter.server;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import no.ntnu.ime.xsltgenerator.server.analyzer.AnalyzerScoringUtilities;
import no.ntnu.ime.xsltgenerator.server.parser.identifiers.OrganizationElementTypes;

import org.junit.Test;


public class AnalyzerScoringUtilitiesTest {

	@Test
	public void getScoreBoardForPathShouldReturnHashWithScoresGroupedByType(){
		HashMap<String, ArrayList<OrganizationElementTypes>> count = new HashMap<String, ArrayList<OrganizationElementTypes>>();
		count.put("/foo/bar", new ArrayList<OrganizationElementTypes>(Arrays.asList(OrganizationElementTypes.ORGANIZATION_ID, OrganizationElementTypes.ORGANIZATION_ID, OrganizationElementTypes.ORGANIZATION_NAME)));
		HashMap<OrganizationElementTypes, Integer> expectedScoreBoard = new HashMap<OrganizationElementTypes, Integer>();
		expectedScoreBoard.put(OrganizationElementTypes.ORGANIZATION_ID, 2);
		expectedScoreBoard.put(OrganizationElementTypes.ORGANIZATION_NAME, 1);
		assertEquals(expectedScoreBoard, AnalyzerScoringUtilities.getScoreBoardForPath("/foo/bar", count));
	}
	
	@Test
	public void getScoreBoardForPathShouldReturnNullIfPathIsNotFound(){
		HashMap<String, ArrayList<OrganizationElementTypes>> count = new HashMap<String, ArrayList<OrganizationElementTypes>>();
		count.put("/foo/bar", new ArrayList<OrganizationElementTypes>(Arrays.asList(OrganizationElementTypes.ORGANIZATION_ID)));
		assertNull(AnalyzerScoringUtilities.getScoreBoardForPath("/i/dont/exist", count));
	}
	
	
	@Test
	public void getBestTypeMatchForPathShouldReturnNullIfPathNotFound(){
		HashMap<String, ArrayList<OrganizationElementTypes>> count = new HashMap<String, ArrayList<OrganizationElementTypes>>();
		count.put("/foo/bar", new ArrayList<OrganizationElementTypes>(Arrays.asList(OrganizationElementTypes.ORGANIZATION_ID)));
		assertNull(AnalyzerScoringUtilities.getBestTypeMatchForPath("/i/dont/exist", count));
	}
	
	@Test
	public void getBestTypeMatchForPathShouldReturnTheMostOccuringTypeForAGivenPath(){
		HashMap<String, ArrayList<OrganizationElementTypes>> count = new HashMap<String, ArrayList<OrganizationElementTypes>>();
		count.put("/foo/bar", new ArrayList<OrganizationElementTypes>(Arrays.asList(OrganizationElementTypes.ORGANIZATION_ID, OrganizationElementTypes.ORGANIZATION_ID, OrganizationElementTypes.ORGANIZATION_NAME)));
		assertEquals(OrganizationElementTypes.ORGANIZATION_ID, AnalyzerScoringUtilities.getBestTypeMatchForPath("/foo/bar", count));
	}
	@Test
	public void getBestTypeMatchForPathShouldReturnTheMostOccuringTypeForAGivenPathIgnoringNulls(){
		HashMap<String, ArrayList<OrganizationElementTypes>> count = new HashMap<String, ArrayList<OrganizationElementTypes>>();
		count.put("/foo/bar", new ArrayList<OrganizationElementTypes>(Arrays.asList(null,null,null,null,OrganizationElementTypes.ORGANIZATION_ID, OrganizationElementTypes.ORGANIZATION_ID, OrganizationElementTypes.ORGANIZATION_NAME)));
		assertEquals(OrganizationElementTypes.ORGANIZATION_ID, AnalyzerScoringUtilities.getBestTypeMatchForPath("/foo/bar", count));
	}
}
