package no.ntnu.ime.xmlconverter.server;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import no.ntnu.ime.xsltgenerator.client.exceptions.ValidSourceException;
import no.ntnu.ime.xsltgenerator.server.XMLSourceParserServiceImpl;

import org.junit.Before;
import org.junit.Test;

public class XMLSourceParserServiceImplTest {

	private XMLSourceParserServiceImpl xmlsps;

	@Before
	public void setUp(){
		xmlsps = new XMLSourceParserServiceImpl();
	}
		
	@Test
	public void testIsValidSourceURIShouldReturnFalseOnEmptyString() throws ValidSourceException {
		assertFalse(xmlsps.isValidSourceURI(""));
	}
	
	@Test
	public void testIsValidSourceURIShouldReturnTrueOnHttpAddress() throws ValidSourceException {
		assertTrue(xmlsps.isValidSourceURI("http://google.com"));
	}
	@Test
	public void testIsValidSourceURIShouldReturnTrueOnHttpsAddress() throws ValidSourceException {
		assertTrue(xmlsps.isValidSourceURI("https://encrypted.google.com/"));
	}
	
	@Test
	public void testIsValidSourceURIShouldReturnTrueOnFTPAddress() throws ValidSourceException {
		assertTrue(xmlsps.isValidSourceURI("ftp://ftp.uio.no/"));
	}
	
	@Test
	public void testIsValidSourceURIShouldReturnTrueOnFTPAddressWithUsernameAndPassword() throws ValidSourceException {
		assertTrue(xmlsps.isValidSourceURI("ftp://Anonymous:foobar@ftp.uio.no"));
	}
	
	@Test(expected=ValidSourceException.class)
	public void testIsValidSourceURIShouldRaiseValidSourceExceptionOnMalformedURL() throws ValidSourceException{
		xmlsps.isValidSourceURI("htttp://google.com");
	}
	
	@Test(expected=ValidSourceException.class)
	public void testIsValidSourceURIShouldRaiseValidSourceExceptionOnNotConnectableAddress() throws ValidSourceException {
		xmlsps.isValidSourceURI("http://i.will.never.exist.klette.us/source.xml");
	}

}
