package no.ntnu.ime.libxsltgenerator.analyzer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.apache.commons.lang.StringUtils;

public class AnalyzerScoringUtilities {
	private AnalyzerScoringUtilities() {
	}

	public static <T> TreeMap<String, HashMap<String, ArrayList<T>>> getRankedResultsForScoreChart(
			HashMap<String, ArrayList<T>> scoreChart) {
		TreeMap<String, ArrayList<T>> sortedMap = new TreeMap<String, ArrayList<T>>();

		for (Entry<String, ArrayList<T>> e : scoreChart.entrySet()) {
			boolean first = true;
			String finalKey = "";
			ArrayList<String> keys = new ArrayList<String>();
			for (String key : e.getKey().split("\\|")) {
				keys.add(key);
				finalKey = StringUtils.join(keys, "|");
				if (!sortedMap.containsKey(finalKey)) {
					sortedMap.put(finalKey, new ArrayList<T>());
				}
				if (!first) {
					sortedMap.get(finalKey).addAll(e.getValue());
				} else {
					first = false;
				}
			}
		}
		removeNullValues(sortedMap);

		return null;
	}

	public static List<String> decomposePathToParts(String path, String pathSeparator){
		String[] parts = StringUtils.split(path, pathSeparator);
		return Arrays.asList(parts);
	}

	public static <T> HashMap<T, Integer> getScoreBoardForPath(String path,
			HashMap<String, ArrayList<T>> count) {
		HashMap<T, Integer> scoreBoard = new HashMap<T, Integer>();
		// Make a copy of the count, so we can modify it at will.
		HashMap<String, ArrayList<T>> newCount = safeScoreMapCopy(count);
		removeNullValues(newCount);
		
		if (!newCount.containsKey(path)) {
			return null;
		} else {

			for (T item : newCount.get(path)) {
				if (!scoreBoard.containsKey(item)) {
					scoreBoard.put(item, Integer.valueOf(1));
				} else {
					scoreBoard.put(item, scoreBoard.get(item) + 1);
				}
			}
		}

		return scoreBoard;
	}

	public static <T> T getBestTypeMatchForPath(String path,
			HashMap<String, ArrayList<T>> count) {
		HashMap<T, Integer> scoreBoard = getScoreBoardForPath(path, count);
		if (scoreBoard == null) {
			return null;
		}
		T highestType = null;
		Integer highestScore = 0;
		for (Entry<T, Integer> entry : scoreBoard.entrySet()) {
			if (entry.getValue() > highestScore) {
				highestType = entry.getKey();
				highestScore = entry.getValue();
			}
		}

		return highestType;
	}

	private static <T> HashMap<String, ArrayList<T>> safeScoreMapCopy(
			Map<String, ArrayList<T>> count) {
		HashMap<String, ArrayList<T>> newScore = new HashMap<String, ArrayList<T>>();
		Iterator<Map.Entry<String, ArrayList<T>>> countIterator = count.entrySet().iterator();
		while (countIterator.hasNext()) {
			Map.Entry<String, ArrayList<T>> e = countIterator.next();
			newScore.put(e.getKey(), new ArrayList<T>());
			Iterator<T> arrayIterator = e.getValue().iterator();
			while (arrayIterator.hasNext()) {
				newScore.get(e.getKey()).add(arrayIterator.next());
			}
		}
		return newScore;
	}
	private static <T> void removeNullValues(Map<String, ArrayList<T>> map) {
		// Reduce score for each null-identifier
		for (Entry<String, ArrayList<T>> e : map.entrySet()) {
			ArrayList<T> pei = e.getValue();
			while (pei.remove(null)) {
			}
		}
	}

}
