package no.ntnu.ime.libxsltgenerator.generator;

import java.util.ArrayList;

import no.ntnu.ime.libxsltgenerator.models.SaxParserResultData;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.PersonElementTypes;

public class XSLTFactory {
	
	private GeneratedXSLT transformation;
	private ArrayList<RelationTemplate> templatesToCall = new ArrayList<RelationTemplate>();
	private ArrayList<RelationTemplate> relations;
	public XSLTFactory() {
		transformation = new GeneratedXSLT();
		relations = new ArrayList<RelationTemplate>();
	}

	public GeneratedXSLT getTransformation() {
		return transformation;
	}
	
	/**
	 * For each identified relationship type
	 * 		create template with the identified matcher and a unique name (rel_templ_%d)
	 *      add template name to the templatesToCallList for execution in the person element
	 *      
	 * @param memberGroup
	 * @return
	 */
	
	public void addMembership(String subject, String object){
		RelationTemplate relTemplate = new RelationTemplate(subject, object);
		relations.add(relTemplate);
		transformation.addPart(relTemplate.toKeyString());
	}
	
	public String getMembershipTemplates(){
		StringBuilder buffer = new StringBuilder();
		for (int i = 0; i < templatesToCall.size(); i++){
			buffer.append(templatesToCall.get(i).toString(i, ""));
		}
		return buffer.toString();
	}
	

	public void addPerson(String xpath, ArrayList<SaxParserResultData> arrayList) {
		StringBuffer templateBuilder = new StringBuffer();
	templateBuilder.append(String.format(
		        "\n\n<xsl:template match=\"%s\">\n<of:person>", xpath));
		for (RelationTemplate rel : relations) {
			templateBuilder.append(rel.getInlineMatcher());

		}
		for (SaxParserResultData dataEntry : arrayList){
			String type = dataEntry.getType();
			PersonElementTypes ptype = PersonElementTypes.descriptionToEnumValue(type);
			String value = dataEntry.getQuery();
			if (ptype != null) {
			switch(ptype){
			case SSN:
		    templateBuilder
			    .append(String
				    .format("<of:norEduPersonNIN><xsl:value-of select=\"%s\"/></of:norEduPersonNIN>\n",
					    value));
				
				break;
			case GENERIC_NAME_NODE:
		    templateBuilder
			    .append(String
				    .format("<of:norEduPersonLegalName><xsl:value-of select=\"%s\"/></of:norEduPersonLegalName>\n",
					    value));
		    templateBuilder.append(String.format(
			    "<of:cn><xsl:value-of select=\"%s\"/></of:cn>\n",
			    value));
		    templateBuilder
			    .append(String
				    .format("<of:displayName><xsl:value-of select=\"%s\"/></of:displayName>\n",
					    value));
				break;
			case GIVEN_NAME:
		    templateBuilder
			    .append(String
				    .format("<of:givenName><xsl:value-of select=\"%s\"/></of:givenName>\n",
					    value));
				break;
			case FIRST_NAME:
				templateBuilder.append(String
				    .format("<of:fName><xsl:value-of select=\"%s\"/></of:fName>\n",
					    value));
				break;
			case LAST_NAME:
				templateBuilder.append(String
				    .format("<of:lName><xsl:value-of select=\"%s\"/></of:lName>\n",
					    value));
				break;
			case BIRTH_NODE:
				templateBuilder.append(String
				    .format("<of:birth><xsl:value-of select=\"%s\"/></of:birth>\n",
					    value));
				break;
			case COUNTRY:
				templateBuilder.append(String
				    .format("<of:country><xsl:value-of select=\"%s\"/></of:country>\n",
					    value));
				break;
			case CITY:
				templateBuilder.append(String
				    .format("<of:city><xsl:value-of select=\"%s\"/></of:city>\n",
					    value));
				break;
			case GENDER:
				templateBuilder.append(String
				    .format("<of:gender><xsl:value-of select=\"%s\"/></of:gender>\n",
					    value));
				break;
			case STREET:
				templateBuilder.append(String
				    .format("<of:address><xsl:value-of select=\"%s\"/></of:address>\n",
					    value));
				break;
			case MIDDLE_NAME:
				templateBuilder.append(String
				    .format("<of:mName><xsl:value-of select=\"%s\"/></of:mName>\n",
					    value));
				break;
			case ZIP_CODE:
				templateBuilder.append(String
				    .format("<of:zipcode><xsl:value-of select=\"%s\"/></of:zipcode>\n",
					    value));
				break;
			case GENERIC_ADDRESS_NODE:
				templateBuilder.append(String
				    .format("<of:genericAddress><xsl:value-of select=\"%s\"/></of:genericAddress>\n",
					    value));
				break;
			}
	
			 
			}
		}
	templateBuilder.append("</of:person>\n</xsl:template>");
		transformation.addPart(templateBuilder.toString());
	}

}
