package no.ntnu.ime.libxsltgenerator.models;

import java.util.ArrayList;
import java.util.List;

public class Relation {
	private ArrayList<String[]> roles = new ArrayList<String[]>();
	private RelationType relationType;
	
	private String ou_identifier = "";
	private String org_identifier = "";
	private String group_identifier = "";
	private ArrayList<String> targets = new ArrayList<String>();
	
	public Relation(RelationType type, String identifier, List<String[]> roles){
		switch(type){
			case PERSON_IN_GROUP:
				group_identifier = identifier;
				break;
			case PERSON_IN_ORG:
				org_identifier = identifier;
				break;
			case PERSON_IN_OU:
				ou_identifier = identifier;
			default:
				throw new IllegalArgumentException("invalid RelationType");
		}
		relationType = type;
		this.roles.addAll(roles);
		
		assert(isValid());
	}
	
	public String getIdentifier(){
		switch(relationType){
		case PERSON_IN_GROUP:
			return group_identifier;
		
		case PERSON_IN_ORG:
			return org_identifier;
		
		case PERSON_IN_OU:
			return ou_identifier;
		default:
			return null;
	}
	}
	
	public void addTarget(String target){
		this.targets.add(target);
	}
	
	private boolean isValid(){
		if (relationType == null)
			return false;
		
		if (relationType == RelationType.PERSON_IN_GROUP){
			return ou_identifier.isEmpty() && org_identifier.isEmpty() && !group_identifier.isEmpty();
		}
		else if (relationType == RelationType.PERSON_IN_ORG){
			return ou_identifier.isEmpty() && group_identifier.isEmpty() && !org_identifier.isEmpty();
		} else if (relationType == RelationType.PERSON_IN_OU){
			return group_identifier.isEmpty() && org_identifier.isEmpty() && !ou_identifier.isEmpty();
		}
		return false;
	}
	
	public String personGroupMembershipsXslt(){
		
		return "<of:eduPersonEntitlement>";
	}
	
	public String personAffiliationXslt(){
		StringBuilder testCases = new StringBuilder();
		for (String[] relRole : roles){
			String attr = relRole[1].substring(1, relRole[1].length()-1);
			String role = relRole[0];
			
			testCases.append("<xsl:when test=\"" + attr + "\">" + role + "</xsl:when>\n");
			
		}
		return "<of:eduPersonAffiliation>" +
			   ""+
			   "/relation/subject/org/orgid[../../object/personid/text()=personid[@foo=fnr]" +
               " <xsl:choose> " +
               testCases.toString() +
               "     <xsl:otherwise /> " +
               " </xsl:choose>" +
               "</of:eduPersonAffiliation>";
	}
	
}
