package no.ntnu.ime.libxsltgenerator.parser.identifiers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.regex.Pattern;

public abstract class ElementAnalyzer<T> {
	
	private HashMap<T, Collection<Pattern>> matchers = new HashMap<T, Collection<Pattern>>();
	
	protected void addMatcherPairs(T type, Collection<String> patterns){
		ArrayList<Pattern> compiledPatterns = new ArrayList<Pattern>(patterns.size());
		for (String p : patterns){
			compiledPatterns.add(Pattern.compile(p, Pattern.CASE_INSENSITIVE));
		}
		
		if(matchers.containsKey(type)){
			matchers.get(type).addAll(compiledPatterns);
		} else {
			matchers.put(type, compiledPatterns);
		}
	}
	
	public T  match(final String elementName){
		for (Entry<T, Collection<Pattern>> entry : matchers.entrySet()){
			for (Pattern pattern : entry.getValue()){
				if (pattern.matcher(elementName).matches()){
					return entry.getKey();
				}
			}
		}
		return null;
	}
	
	protected static boolean checkAgainstRegexpList(final String elementName, String[] regexpList) {
		for (String regexp : regexpList){
			if (Pattern.compile(regexp, Pattern.CASE_INSENSITIVE).matcher(elementName).matches()){
				return true;
			}
		}
		return false;
	}
}