package no.ntnu.ime.libxsltgenerator.parser.identifiers;

/**
 * Utility class for identifying organizational values.
 * 
 * The class can not be instanceiated and all methods are static.
 * 
 * @author Kristian Klette <kriskl@stud.ntnu.no>
 *
 */
public class OrganizationNodeIdentifiers {
	private static final int MOD_PARAM = 11;
	private static final int INVALID_CHECKSUM = 10;
	private static final int VALID_ORGID_LENGTH = 9;

	private OrganizationNodeIdentifiers(){}
	
	/**
	 * Weights for checksum generation.
	 * See http://www.brreg.no/samordning/organisasjonsnummer.html (Norwegian) for more information.
	 */
	private static final int[] WEIGTHS = {3,2,7,6,5,4,3,2};
	
	/**
	 * Returns true if the given string is a valid Norwegian business identification number.
	 * A valid number has nine digits where the last one is a checksum.
	 * @param orgIdent
	 * @return
	 */
	public static boolean isValidOrganizationNumber(String orgIdent){
		if (orgIdent == null || orgIdent.length() != VALID_ORGID_LENGTH){
			return false;
		}
		
		int checksum = 0;
		try {
			for (int position = 0; position < WEIGTHS.length; position++){
				checksum += Integer.parseInt(orgIdent.substring(position, position+1)) * WEIGTHS[position];
			}
		} catch (NumberFormatException e){
			return false;
		}
		checksum = MOD_PARAM - (checksum % MOD_PARAM);
		// Set checksum to zero if the modulo returns zero as according to spec.
		if (checksum == MOD_PARAM){
			checksum = 0;
		}
		
		return (checksum != INVALID_CHECKSUM && orgIdent.endsWith(String.valueOf(checksum)));
	}
}
