package no.ntnu.ime.libxsltgenerator.parser.identifiers;

public class PersonNodeContentAnalyzer {
	// Prevent instantiation
	private PersonNodeContentAnalyzer() {
	}
	private static final int[] CHECKSUM_WEIGTHS_ROUND_FIRST_DIGIT =  {3,7,6,1,8,9,4,5,2};
	private static final int[] CHECKSUM_WEIGTHS_ROUND_SECOND_DIGIT = {5,4,3,2,7,6,5,4,3,2};
	/**
	 * Returns true if the given string is a valid Norwegian SSN.
	 * 
	 * @param identString
	 * @return
	 */
	public static boolean isPersonId(String identString) {
		
		if (identString == null || identString.length() != 11){
			return false;
		}
		
		try {
			return (hasValidChecksumForWeigths(identString, CHECKSUM_WEIGTHS_ROUND_FIRST_DIGIT) &&
					hasValidChecksumForWeigths(identString, CHECKSUM_WEIGTHS_ROUND_SECOND_DIGIT));
			
		} catch (NumberFormatException e) {
			return false;
		}
	}
	/**
	 * Calculates the checksum-digit for the give weigths and checks if it matches the one given in identString.
	 * 
	 * @param identString
	 * @param weigths
	 * @return booolean
	 */
	private static boolean hasValidChecksumForWeigths(String identString, final int[] weigths) {
		int checksum = 0;
		for (int index = 0; index < weigths.length; index++) {
			int curNum = Integer.parseInt(String.valueOf(identString.charAt(index)));
			checksum += weigths[index] * curNum;
		}
		checksum = 11 - (checksum % 11);
		if (checksum == 11){
			checksum = 0;
		}
		if (identString.charAt(weigths.length) != String.valueOf(checksum).charAt(0)){
			return false;
		}
		return true;
	}
	
	public static boolean isKnownNorwegianGivenName(String candidateString){
		for (String known : NameLookup.getNorwegianGivenNames()){
			if(candidateString.trim().equals(known)){
				return true;
			}
		}
		return false;
	}

}
