package no.ntnu.ime.libxsltgenerator.parsers;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.TreeMap;

import javax.xml.xpath.XPathExpressionException;

import no.ntnu.ime.libxsltgenerator.models.SaxParserResult;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.GroupElementAnalyzer;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.GroupElementTypes;
import no.ntnu.ime.libxsltgenerator.parsers.interfaces.SAXParser;

import org.apache.commons.lang.StringUtils;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;

public class GroupSAXParser extends SaxParserDefaults<GroupElementTypes> implements SAXParser {

	private static final String XPATH_SEPERATOR = "/";
	
	ArrayList<String> stack = new ArrayList<String>();
	HashMap<String, ArrayList<GroupElementTypes>> count = new HashMap<String, ArrayList<GroupElementTypes>>();
	HashMap<String, GroupElementTypes> locationToTypeMap = new HashMap<String, GroupElementTypes>();
	GroupElementAnalyzer elementAnalyzer = new GroupElementAnalyzer();
	
	private GroupElementTypes currentType = null;
	private String maxGroupKey;
	
	public void addCount(GroupElementTypes type){
		String key = StringUtils.join(stack, XPATH_SEPERATOR);
		if (count.containsKey(key)){
			count.get(key).add(type);
		} else {
			count.put(key, new ArrayList<GroupElementTypes>(Arrays.asList(type)));
		}
	}
	


	
	public void parseCharacterData(String data) {
	

	}

	
	public void parseStartElement(String uri, String localName, String qName,
			Attributes attributes) {
		stack.add(localName);
		currentType = elementAnalyzer.match(localName);

	}

	
	public void parseEndElement(String uri, String localName, String qName) {
		if (currentType != null){
			locationToTypeMap.put(StringUtils.join(stack, XPATH_SEPERATOR), currentType);
		}
		
		stack.remove(localName);
		addCount(currentType);
		currentType = null;
	}

	
	public void endDocumentParsing() {	
		TreeMap<String, ArrayList<GroupElementTypes>> sortedMap = new TreeMap<String, ArrayList<GroupElementTypes>>();
		for (Entry<String, ArrayList<GroupElementTypes>> e : count.entrySet()){
			boolean first = true;
			String finalKey = "";
			ArrayList<String> keys = new ArrayList<String>();
			for (String key : e.getKey().split("\\" + XPATH_SEPERATOR)){
				keys.add(key);
				finalKey = StringUtils.join(keys, XPATH_SEPERATOR);
				if (!sortedMap.containsKey(finalKey)){
					sortedMap.put(finalKey, new ArrayList<GroupElementTypes>());
				}
				if (!first){
					sortedMap.get(finalKey).addAll(e.getValue());
				} else {
					first = false;
				}
			}
		}
		// Reduce score for each null-identifier
		for (Entry<String, ArrayList<GroupElementTypes>> e : sortedMap.entrySet()){
			ArrayList<GroupElementTypes> pei = e.getValue();
			while(pei.remove(null)){}
		}
		int maxScore = 0;
		maxGroupKey = "";
		for (Entry<String, ArrayList<GroupElementTypes>> e : sortedMap.entrySet()){
			if (e.getValue().size() > 0){
				if (e.getValue().size() > maxScore){
					maxScore = e.getValue().size();
					maxGroupKey = e.getKey();
				}
			}
		}
	
		HashMap<String, GroupElementTypes> locMap = new HashMap<String, GroupElementTypes>();
		for (String locKey : locationToTypeMap.keySet()){
			if (locKey.startsWith(maxGroupKey)){
				locMap.put(locKey, locationToTypeMap.get(locKey));
			}
		}
		locationToTypeMap = locMap;

	}
	
	
	public SaxParserResult getParserResult(InputSource source) {
		try {
			return getFirstMatch(source, maxGroupKey, locationToTypeMap);
		} catch (XPathExpressionException e) {
			e.printStackTrace();
			return null;
		}
	}



	public void startDocumentParsing() {
		// TODO Auto-generated method stub
		
	}

}
