package no.ntnu.ime.libxsltgenerator.parsers;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.TreeMap;

import javax.xml.xpath.XPathExpressionException;

import no.ntnu.ime.libxsltgenerator.models.SaxParserResult;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.OrganizationElementAnalyzer;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.OrganizationElementTypes;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.OrganizationNodeIdentifiers;
import no.ntnu.ime.libxsltgenerator.parsers.interfaces.SAXParser;

import org.apache.commons.lang.StringUtils;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;

public class OrganizationSAXParser extends SaxParserDefaults<OrganizationElementTypes> implements SAXParser {
	ArrayList<String> stack = new ArrayList<String>();
	HashMap<String, ArrayList<OrganizationElementTypes>> count = new HashMap<String, ArrayList<OrganizationElementTypes>>();
	HashMap<String, OrganizationElementTypes> locationToTypeMap = new HashMap<String, OrganizationElementTypes>();
	OrganizationElementAnalyzer elementAnalyzer = new OrganizationElementAnalyzer();
	
	private OrganizationElementTypes currentType = null;
	private String maxOrganizationKey;
	
	public void addCount(OrganizationElementTypes type){
		String key = StringUtils.join(stack, "|");
		if (count.containsKey(key)){
			count.get(key).add(type);
		} else {
			count.put(key, new ArrayList<OrganizationElementTypes>(Arrays.asList(type)));
		}
	}
	
	public void parseCharacterData(String data) {
		if (OrganizationNodeIdentifiers.isValidOrganizationNumber(data)){
			currentType = OrganizationElementTypes.ORGANIZATION_ID;
		}
	}

	
	public void parseStartElement(String uri, String localName, String qName,
			Attributes attributes) {
		stack.add(localName);
		currentType = elementAnalyzer.match(localName);

	}

	
	public void parseEndElement(String uri, String localName, String qName) {
		if (currentType != null){
			locationToTypeMap.put(StringUtils.join(stack, "|"), currentType);
		}
		
		stack.remove(localName);
		addCount(currentType);
		currentType = null;

	}

	public void endDocumentParsing() {
		TreeMap<String, ArrayList<OrganizationElementTypes>> sortedMap = new TreeMap<String, ArrayList<OrganizationElementTypes>>();
		for (Entry<String, ArrayList<OrganizationElementTypes>> e : count.entrySet()){
			boolean first = true;
			String finalKey = "";
			ArrayList<String> keys = new ArrayList<String>();
			for (String key : e.getKey().split("\\|")){
				keys.add(key);
				finalKey = StringUtils.join(keys, "|");
				if (!sortedMap.containsKey(finalKey)){
					sortedMap.put(finalKey, new ArrayList<OrganizationElementTypes>());
				}
				if (!first){
					sortedMap.get(finalKey).addAll(e.getValue());
				} else {
					first = false;
				}
			}
		}
		// Reduce score for each null-identifier
		for (Entry<String, ArrayList<OrganizationElementTypes>> e : sortedMap.entrySet()){
			ArrayList<OrganizationElementTypes> pei = e.getValue();
			while(pei.remove(null)){}
		}
		int maxScore = 0;
		maxOrganizationKey = "";
		for (Entry<String, ArrayList<OrganizationElementTypes>> e : sortedMap.entrySet()){
			if (e.getValue().size() > 0){
				if (e.getValue().size() > maxScore){
					maxScore = e.getValue().size();
					maxOrganizationKey = e.getKey();
				}
			}
		}
	
		HashMap<String, OrganizationElementTypes> locMap = new HashMap<String, OrganizationElementTypes>();
		for (String locKey : locationToTypeMap.keySet()){
			if (locKey.startsWith(maxOrganizationKey)){
				locMap.put(locKey, locationToTypeMap.get(locKey));
			}
		}
		locationToTypeMap = locMap;


	}

	public SaxParserResult getParserResult(InputSource source) {
		try {
			return getFirstMatch(source, maxOrganizationKey, locationToTypeMap);
		} catch (XPathExpressionException e) {
			e.printStackTrace();
			return null;
		}
	}

	public void startDocumentParsing() {
		// TODO Auto-generated method stub
		
	}


}
