/**
 * 
 */
package no.ntnu.ime.libxsltgenerator.parsers;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.TreeMap;

import javax.xml.xpath.XPathExpressionException;

import no.ntnu.ime.libxsltgenerator.models.SaxParserResult;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.PersonElementAnalyzer;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.PersonElementTypes;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.PersonNodeContentAnalyzer;
import no.ntnu.ime.libxsltgenerator.parsers.interfaces.SAXParser;

import org.apache.commons.lang.StringUtils;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;

/**
 * @author klette
 *
 */
public class PersonSAXParser extends SaxParserDefaults<PersonElementTypes> implements SAXParser{
	
	ArrayList<String> stack = new ArrayList<String>();
	HashMap<String, ArrayList<PersonElementTypes>> count = new HashMap<String, ArrayList<PersonElementTypes>>();
	HashMap<String, PersonElementTypes> locationToTypeMap = new HashMap<String, PersonElementTypes>();
	PersonElementAnalyzer elementAnalyzer = new PersonElementAnalyzer();
	
	private PersonElementTypes currentType = null;
	private String maxPersonKey;
	
	public void addCount(PersonElementTypes type){
		String key = StringUtils.join(stack, "|");
		if (count.containsKey(key)){
			count.get(key).add(type);
		} else {
			count.put(key, new ArrayList<PersonElementTypes>(Arrays.asList(type)));
		}
	}

	/* (non-Javadoc)
	 * @see no.ntnu.ime.xmlconverter.server.SAXParser#startDocumentParsing()
	 */
	
	public void startDocumentParsing() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see no.ntnu.ime.xmlconverter.server.SAXParser#parseCharacterData(java.lang.String)
	 */
	
	public void parseCharacterData(String data) {
		if (data.trim().isEmpty())
			return;
		if (PersonNodeContentAnalyzer.isPersonId(data)){
			currentType = PersonElementTypes.SSN;
		} else if (PersonNodeContentAnalyzer.isKnownNorwegianGivenName(data)){
			currentType = PersonElementTypes.GIVEN_NAME;
		}
	}

	/* (non-Javadoc)
	 * @see no.ntnu.ime.xmlconverter.server.SAXParser#parseStartElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	
	public void parseStartElement(String uri, String localName, String qName,
			Attributes attributes) {
		stack.add(localName);
		currentType = elementAnalyzer.match(localName);
	}

	/* (non-Javadoc)
	 * @see no.ntnu.ime.xmlconverter.server.SAXParser#parseEndElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	
	public void parseEndElement(String uri, String localName, String qName) {
		if (currentType != null){
			locationToTypeMap.put(StringUtils.join(stack, "|"), currentType);
		}
		
		stack.remove(localName);
		addCount(currentType);
		currentType = null;
	}

	/* (non-Javadoc)
	 * @see no.ntnu.ime.xmlconverter.server.SAXParser#endDocumentParsing()
	 */
	
	public void endDocumentParsing() {
		TreeMap<String, ArrayList<PersonElementTypes>> sortedMap = new TreeMap<String, ArrayList<PersonElementTypes>>();
		for (Entry<String, ArrayList<PersonElementTypes>> e : count.entrySet()){
			boolean first = true;
			String finalKey = "";
			ArrayList<String> keys = new ArrayList<String>();
			for (String key : e.getKey().split("\\|")){
				keys.add(key);
				finalKey = StringUtils.join(keys, "|");
				if (!sortedMap.containsKey(finalKey)){
					sortedMap.put(finalKey, new ArrayList<PersonElementTypes>());
				}
				if (!first){
					sortedMap.get(finalKey).addAll(e.getValue());
				} else {
					first = false;
				}
			}
		}
		// Reduce score for each null-identifier
		for (Entry<String, ArrayList<PersonElementTypes>> e : sortedMap.entrySet()){
			ArrayList<PersonElementTypes> pei = e.getValue();
			while(pei.remove(null)){}
		}
		int maxScore = 0;
		maxPersonKey = "";
		for (Entry<String, ArrayList<PersonElementTypes>> e : sortedMap.entrySet()){
			if (e.getValue().size() > 0){
				if (e.getValue().size() > maxScore){
					maxScore = e.getValue().size();
					maxPersonKey = e.getKey();
				}
			}
		}
	
		HashMap<String, PersonElementTypes> locMap = new HashMap<String, PersonElementTypes>();
		for (String locKey : locationToTypeMap.keySet()){
			if (locKey.startsWith(maxPersonKey)){
				locMap.put(locKey, locationToTypeMap.get(locKey));
			}
		}
		locationToTypeMap = locMap;

	}
	
	public SaxParserResult getParserResult(InputSource source) {
		try {
			return getFirstMatch(source, maxPersonKey, locationToTypeMap);
		} catch (XPathExpressionException e) {
			e.printStackTrace();
			return null;
		}
	}

}
