package no.ntnu.ime.libxsltgenerator.parsers;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import no.ntnu.ime.libxsltgenerator.models.SaxParserResult;
import no.ntnu.ime.libxsltgenerator.models.SaxParserResultData;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.RelationElementAnalyzer;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.RelationElementTypes;
import no.ntnu.ime.libxsltgenerator.parsers.interfaces.SAXParser;

import org.apache.commons.lang.StringUtils;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;

public class RelationSAXParser extends SaxParserDefaults<RelationElementTypes>
		implements SAXParser {
	Logger log = Logger.getLogger(getClass().getName());
	
	private static final String XPATH_SEPERATOR = "/";

	ArrayList<String> stack = new ArrayList<String>();
	HashMap<String, ArrayList<RelationElementTypes>> count = new HashMap<String, ArrayList<RelationElementTypes>>();
	HashMap<String, RelationElementTypes> locationToTypeMap = new HashMap<String, RelationElementTypes>();
	RelationElementAnalyzer elementAnalyzer = new RelationElementAnalyzer();

	private RelationElementTypes currentType = null;
	private String maxKey;

	public void addCount(RelationElementTypes type) {
		String key = StringUtils.join(stack, XPATH_SEPERATOR);
		if (count.containsKey(key)) {
			count.get(key).add(type);
		} else {
			count.put(key,
					new ArrayList<RelationElementTypes>(Arrays.asList(type)));
		}
	}

	public void startDocumentParsing() {
	}

	public void parseCharacterData(String data) {

	}

	public void parseStartElement(String uri, String localName, String qName,
			Attributes attributes) {
		stack.add(localName);
		currentType = elementAnalyzer.match(localName);

	}

	public void parseEndElement(String uri, String localName, String qName) {
		if (currentType != null) {
			locationToTypeMap.put(StringUtils.join(stack, XPATH_SEPERATOR),
					currentType);
			log.fine("Found " + currentType.description + " at "
					+ StringUtils.join(stack, XPATH_SEPERATOR));
		}

		stack.remove(localName);
		addCount(currentType);
		currentType = null;
	}

	public void endDocumentParsing() {
		TreeMap<String, ArrayList<RelationElementTypes>> sortedMap = new TreeMap<String, ArrayList<RelationElementTypes>>();
		for (Entry<String, ArrayList<RelationElementTypes>> e : count
				.entrySet()) {
			boolean first = true;
			String finalKey = "";
			ArrayList<String> keys = new ArrayList<String>();
			for (String key : e.getKey().split("\\" + XPATH_SEPERATOR)) {
				keys.add(key);
				finalKey = StringUtils.join(keys, XPATH_SEPERATOR);
				if (!sortedMap.containsKey(finalKey)) {
					sortedMap.put(finalKey,
							new ArrayList<RelationElementTypes>());
				}
				if (!first) {
					sortedMap.get(finalKey).addAll(e.getValue());
				} else {
					first = false;
				}
			}
		}
		// Reduce score for each null-identifier
		for (Entry<String, ArrayList<RelationElementTypes>> e : sortedMap
				.entrySet()) {
			ArrayList<RelationElementTypes> pei = e.getValue();
			while (pei.remove(null)) {
			}
		}
		int maxScore = 0;
		maxKey = "";
		for (Entry<String, ArrayList<RelationElementTypes>> e : sortedMap
				.entrySet()) {
			if (e.getValue().size() > 0) {
				if (e.getValue().size() > maxScore) {
					maxScore = e.getValue().size();
					maxKey = e.getKey();
				}
			}
		}	

		HashMap<String, RelationElementTypes> locMap = new HashMap<String, RelationElementTypes>();
		for (String locKey : locationToTypeMap.keySet()) {
			if (locKey.startsWith(maxKey)) {
				locMap.put(locKey, locationToTypeMap.get(locKey));
			}
		}
		locationToTypeMap = locMap;

	}

	public List<SaxParserResult> getAllResults(File file) throws Exception {
		List<SaxParserResult> results = new ArrayList<SaxParserResult>();
		TreeMap<String, ArrayList<RelationElementTypes>> sortedMap = new TreeMap<String, ArrayList<RelationElementTypes>>();
		for (Entry<String, ArrayList<RelationElementTypes>> e : count
				.entrySet()) {
			boolean first = true;
			String finalKey = "";
			ArrayList<String> keys = new ArrayList<String>();
			for (String key : e.getKey().split("\\" + XPATH_SEPERATOR)) {
				keys.add(key);
				finalKey = StringUtils.join(keys, XPATH_SEPERATOR);
				if (!sortedMap.containsKey(finalKey)) {
					sortedMap.put(finalKey,
							new ArrayList<RelationElementTypes>());
				}
				if (!first) {
					sortedMap.get(finalKey).addAll(e.getValue());
				} else {
					first = false;
				}
			}
		}
		// Reduce score for each null-identifier
		for (Entry<String, ArrayList<RelationElementTypes>> e : sortedMap
				.entrySet()) {
			ArrayList<RelationElementTypes> pei = e.getValue();
			while (pei.remove(null)) {
			}
		}

		for (Entry<String, ArrayList<RelationElementTypes>> e : sortedMap
				.entrySet()) {
			if (e.getValue().size() > 0) {

				HashMap<String, RelationElementTypes> locMap = new HashMap<String, RelationElementTypes>();
				for (String locKey : locationToTypeMap.keySet()) {
					if (locKey.startsWith(e.getKey())) {
						locMap.put(locKey, locationToTypeMap.get(locKey));
						log.fine(locKey + " : "
								+ locationToTypeMap.get(locKey).description);
					}
				}
				try {
					String xpathKey = e.getKey().replace("|", "/");
					SaxParserResult result = new SaxParserResult(xpathKey, new ArrayList<SaxParserResultData>());
					XPath xpath = XPathFactory.newInstance().newXPath();
					XPathExpression expr = xpath.compile(xpathKey);
					Node parentNode = (Node) expr.evaluate(new InputSource(new InputStreamReader(new FileInputStream(file))), XPathConstants.NODE);
										
					for (Entry<String, RelationElementTypes> entry : locMap.entrySet()) {
						String query = entry.getKey().replace("|", "/").substring(xpathKey.length() + 1);
						XPathExpression queryExpr = xpath.compile(query);

						NodeList nodelist = (NodeList) queryExpr.evaluate(parentNode, XPathConstants.NODESET);
						for (int i = 0; i < nodelist.getLength(); i++) {
							Node node = nodelist.item(i);

							SaxParserResultData saxParserResultData = new SaxParserResultData(
									node.getTextContent(),
									entry.getValue().toString(),
									query);
							result.getEntries().add(
									saxParserResultData);
						}
					}
					results.add(result);
				} catch (XPathExpressionException e1) {
					e1.printStackTrace();
				}
			}
		}
		return results;

	}

	public SaxParserResult getParserResult(InputSource source) {
		try {
			return getFirstMatch(source, maxKey, locationToTypeMap);
		} catch (XPathExpressionException e) {
			e.printStackTrace();
			return null;
		}
	}

}
