package no.ntnu.ime.libxsltgenerator.parsers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import no.ntnu.ime.libxsltgenerator.models.SaxParserResult;
import no.ntnu.ime.libxsltgenerator.models.SaxParserResultData;
import no.ntnu.ime.libxsltgenerator.parser.identifiers.ElementTypes;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public abstract class SaxParserDefaults<E extends ElementTypes> {
	
	public SaxParserResult getFirstMatch(InputSource source,
			String maxPersonKey, HashMap<String, E> locationToTypeMap)
			throws XPathExpressionException {

		SaxParserResult result = new SaxParserResult(maxPersonKey.replace("|",
				"/"), new ArrayList<SaxParserResultData>());

		XPath xpath = XPathFactory.newInstance().newXPath();
		XPathExpression expr = xpath.compile(maxPersonKey.replace("|", "/"));
		Node parentNode = (Node) expr.evaluate(source, XPathConstants.NODE);

		for (Entry<String, E> entry : locationToTypeMap.entrySet()) {
			String query = entry.getKey().replace("|", "/")
					.substring(maxPersonKey.length() + 1);
			XPathExpression queryExpr = xpath.compile(query);

			NodeList nodelist = (NodeList) queryExpr.evaluate(parentNode, XPathConstants.NODESET);
			for (int i = 0; i < nodelist.getLength(); i++) {
				Node node = nodelist.item(i);

				result.getEntries().add(
						new SaxParserResultData(
								node.getTextContent(),
								entry.getValue().toString(),
								query));
			}
		}
		return result;
	}
	public List<SaxParserResult> getAllMatches(InputSource source,
			List<Map<String, HashMap<String, E>>> data)
			throws XPathExpressionException {
		List<SaxParserResult> results = new ArrayList<SaxParserResult>();
		for (Map<String, HashMap<String, E>> dataEntry : data){
			for (Entry<String, HashMap<String, E>> dataEntryEntry : dataEntry.entrySet()){
			String maxPersonKey = dataEntryEntry.getKey();
			HashMap<String, E> locationToTypeMap = dataEntryEntry.getValue();
			
		SaxParserResult result = new SaxParserResult(maxPersonKey.replace("|",
				"/"), new ArrayList<SaxParserResultData>());

		XPath xpath = XPathFactory.newInstance().newXPath();
		XPathExpression expr = xpath.compile(maxPersonKey.replace("|", "/"));
		Node parentNode = (Node) expr.evaluate(source, XPathConstants.NODE);

		for (Entry<String, E> entry : locationToTypeMap.entrySet()) {
			String query = entry.getKey().replace("|", "/")
					.substring(maxPersonKey.length() + 1);
			XPathExpression queryExpr = xpath.compile(query);

			NodeList nodelist = (NodeList) queryExpr.evaluate(parentNode, XPathConstants.NODESET);
			for (int i = 0; i < nodelist.getLength(); i++) {
				Node node = nodelist.item(i);

				result.getEntries().add(
						new SaxParserResultData(
								node.getTextContent(),
								entry.getValue().toString(),
								query));
			}
		}
		results.add(result);
		}
		}
		return results;
	}

}
